﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Reflection;

namespace Builder
{
    public partial class MainWindow : Form
    {
        public MainWindow()
        {
            InitializeComponent();

            treeView.AllowDrop = true;

            addComponent (typeof (Record));
            addComponent (typeof (Field));

            /*
            Type t = typeof(Button);
            foreach (MethodInfo m in t.GetMethods())
                textBox1.AppendText(m.Name + "\n");
            */
        }

        private void addComponent(Type type)
        {
            Component c = new Component (type);

            int n = componentPage.Controls.Count;
            c.Left = c.Left + n * (c.Width + c.Left);
            componentPage.Controls.Add(c);
        }

        private void quitMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void treeView_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent ("Builder.Component"))
               e.Effect = DragDropEffects.Copy;
        }

        Control findTarget (TreeNode node)
        {
            Control t = null;

            while (node != null && t == null)
            {
                Common c = node.Tag as Common;
                if (c != null)
                    t = c.getArea ();

                node = node.Parent;
            }

            if (t == null)
                t = stage;

            return t;
        }

        private void treeView_DragDrop(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent("Builder.Component"))
            {
                Component c = e.Data.GetData("Builder.Component") as Component;
                Object obj = c.CreateInstance();

                TreeNode node = new TreeNode();
                node.Text = obj.ToString();
                node.Tag = obj;

                Point pt = treeView.PointToClient(new Point(e.X, e.Y));
                TreeNode target = treeView.GetNodeAt(pt);
                if (target != null)
                    target.Nodes.Add(node);
                    // target.Nodes.Insert(0, node);
                else
                    treeView.Nodes.Add(node);

                if (obj is Common)
                {
                    Common p = obj as Common;
                    Control t = findTarget(node.Parent);
                    p.setup(node, t);
                }
                else
                {
                    node.BackColor = c.BackColor;
                }

            }

        }

        private void treeView_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            Object obj = e.Node.Tag;
            propertyGrid.SelectedObject = obj;
        }
    }
}
